import { each } from 'lodash-es';
import { instanceOf } from '../../src/instance-of.func';
import { PropertyWrapperClass } from '@angular-package/core/property-wrapper';
export class ApChangeDetectorClass {
    constructor(component, properties = {}) {
        this.properties = properties;
        this.detection = false;
        this.propertyWrapper = new PropertyWrapperClass();
        if (component) {
            this.find(component).setDetection(component).detectToSetter(component);
        }
        return this;
    }
    get cd() {
        return this._cd;
    }
    detach(component) {
        setTimeout(() => {
            if (this.cd) {
                component[this.cd].detach();
            }
        }, 0);
        this.detection = false;
        return this;
    }
    detect(component, property) {
        if (this.cd) {
            if (property) {
                if (this.properties && this.properties[property] === true) {
                    try {
                        component[this.cd].detectChanges();
                    }
                    catch (e) { }
                }
            }
            else {
                component[this.cd].detectChanges();
            }
        }
        return this;
    }
    detectToSetter(component) {
        if (instanceOf(this, 'properties') && this.properties) {
            this.propertyWrapper.wrap(component, Object.keys(this.properties), (property, sourcePropertyName, source) => {
                if (source) {
                    source['changeDetector'].detect(source, property);
                }
            }, (property, source) => undefined);
        }
        return this;
    }
    reattach(component) {
        setTimeout(() => {
            if (this.cd) {
                component[this.cd].reattach();
            }
        }, 0);
        this.detection = true;
        return this;
    }
    setDetection(component) {
        if (component) {
            if (this.detection === true) {
                this.reattach(component);
            }
            else if (this.detection === false) {
                this.detach(component);
            }
        }
        return this;
    }
    find(component) {
        if (this.cd === undefined) {
            each(component, (ChangeDetectorRefInstance, key) => {
                if (component[key] instanceof Object) {
                    if (instanceOf(component[key], 'detectChanges')) {
                        if (component[key].detectChanges instanceof Function) {
                            this._cd = key;
                            return false;
                        }
                    }
                }
            });
            if (this.cd === undefined) {
                throw new Error(`
          ApChangeDetectorClass: couldn't find ChangeDetectorRef instance.
          Add to constructor "public changeDetectorRef: ChangeDetectorRef".
        `);
            }
        }
        return this;
    }
}
//# sourceMappingURL=change-detector.class.js.map