import { merge } from 'lodash-es';
import { DEFAULT_OPTIONS } from './default_options';
import { ApChangeDetectorClass } from '../change-detector/src/change-detector.class';
export function ApChangeDetection(properties, options) {
    return function (component) {
        options = (options) ? merge(DEFAULT_OPTIONS, options) : DEFAULT_OPTIONS;
        Object.defineProperties(component.prototype, {
            _changeDetector: {
                configurable: false,
                writable: true
            },
            changeDetector: {
                set: function (value) {
                    this._changeDetector = value;
                },
                get: function () {
                    if (this._changeDetector === undefined) {
                        this._changeDetector = new ApChangeDetectorClass(this, Object.assign({}, properties));
                    }
                    return this._changeDetector;
                }
            },
            detection: {
                set(detection) {
                    this.changeDetector.detection = detection;
                    this.changeDetector.setDetection(this);
                },
                get() {
                    return this.changeDetector.detection;
                }
            },
        });
        if (options) {
            if (options.detach) {
                Object.defineProperties(component.prototype, {
                    [`${options.detach}`]: {
                        configurable: false,
                        writable: true,
                        value: function () {
                            this.changeDetector.detach(this);
                        }
                    }
                });
            }
            if (options.detect) {
                Object.defineProperties(component.prototype, {
                    [`${options.detect}`]: {
                        configurable: false,
                        writable: true,
                        value: function (property) {
                            this.changeDetector.detect(this, property);
                        }
                    }
                });
            }
            if (options.properties) {
                Object.defineProperties(component.prototype, {
                    [`${options.properties}`]: {
                        set: function (value) {
                            this.changeDetector.properties = value;
                            this.changeDetector.detect(this);
                            this.changeDetector.detectToSetter(this);
                        },
                        get: function () {
                            return this.changeDetector.properties;
                        }
                    }
                });
            }
            if (options.reattach) {
                Object.defineProperties(component.prototype, {
                    [`${options.reattach}`]: {
                        configurable: false,
                        writable: true,
                        value: function () {
                            this.changeDetector.reattach(this);
                        }
                    }
                });
            }
        }
    };
}
//# sourceMappingURL=change-detection.decorator.js.map